@@ -100,10 +100,13 @@ class BaseOS(object):
         with os.fdopen(os.open(interface_file_path, flags, mode),
                        'w') as text_file:
             text = template_vip.render(
+                consts=consts,
                 interface=primary_interface,
                 vip=vip,
                 vip_ipv6=ip.version is 6,
-                prefix=utils.netmask_to_prefix(netmask),
+                # For ipv6 the netmask is already the prefix
+                prefix=(netmask if ip.version is 6
+                        else utils.netmask_to_prefix(netmask)),
                 broadcast=broadcast,
                 netmask=netmask,
                 gateway=gateway,
@@ -112,6 +115,7 @@ class BaseOS(object):
                 vrrp_ip=vrrp_ip,
                 vrrp_ipv6=vrrp_version is 6,
                 host_routes=render_host_routes,
+                topology=CONF.controller_worker.loadbalancer_topology,
             )
             text_file.write(text)
 
@@ -132,11 +136,12 @@ class BaseOS(object):
 
         with os.fdopen(os.open(interface_file_path, flags, mode),
                        'w') as text_file:
-            text = self._generate_network_file_text(netns_interface, fixed_ips,
-                                                    mtu, template_port)
+            text = self._generate_network_file_text(
+                netns_interface, fixed_ips, mtu, template_port)
             text_file.write(text)
 
-    def _generate_network_file_text(self, netns_interface, fixed_ips, mtu,
+    @classmethod
+    def _generate_network_file_text(cls, netns_interface, fixed_ips, mtu,
                                     template_port):
         text = ''
         if fixed_ips is None:
@@ -154,7 +159,7 @@ class BaseOS(object):
                     broadcast = network.broadcast_address.exploded
                     netmask = (network.prefixlen if ip.version is 6
                                else network.netmask.exploded)
-                    host_routes = self.get_host_routes(fixed_ip)
+                    host_routes = cls.get_host_routes(fixed_ip)
 
                 except ValueError:
                     return webob.Response(
@@ -169,7 +174,8 @@ class BaseOS(object):
                 text = '\n'.join([text, new_text])
         return text
 
-    def get_host_routes(self, fixed_ip):
+    @classmethod
+    def get_host_routes(cls, fixed_ip):
         host_routes = []
         for hr in fixed_ip.get('host_routes', []):
             network = ipaddress.ip_network(
@@ -179,7 +185,8 @@ class BaseOS(object):
             host_routes.append({'network': network, 'gw': hr['nexthop']})
         return host_routes
 
-    def _bring_if_up(self, interface, what):
+    @classmethod
+    def _bring_if_up(cls, interface, what):
         # Note, we are not using pyroute2 for this as it is not /etc/netns
         # aware.
         cmd = ("ip netns exec {ns} ifup {params}".format(
@@ -194,7 +201,8 @@ class BaseOS(object):
                     message='Error plugging {0}'.format(what),
                     details=e.output), status=500))
 
-    def _bring_if_down(self, interface):
+    @classmethod
+    def _bring_if_down(cls, interface):
         # Note, we are not using pyroute2 for this as it is not /etc/netns
         # aware.
         cmd = ("ip netns exec {ns} ifdown {params}".format(
@@ -205,11 +213,14 @@ class BaseOS(object):
             LOG.info('Ignoring failure to ifdown %s due to error: %s %s',
                      interface, e, e.output)
 
-    def bring_interfaces_up(self, ip, primary_interface, secondary_interface):
-        self._bring_if_down(primary_interface)
-        self._bring_if_down(secondary_interface)
-        self._bring_if_up(primary_interface, 'VIP')
-        self._bring_if_up(secondary_interface, 'VIP')
+    @classmethod
+    def bring_interfaces_up(cls, ip, primary_interface, secondary_interface):
+        cls._bring_if_down(primary_interface)
+        if secondary_interface:
+            cls._bring_if_down(secondary_interface)
+        cls._bring_if_up(primary_interface, 'VIP')
+        if secondary_interface:
+            cls._bring_if_up(secondary_interface, 'VIP')
 
     def has_ifup_all(self):
         return True
@@ -373,7 +384,10 @@ class RH(BaseOS):
             netmask, gateway, mtu, vrrp_ip, vrrp_version, render_host_routes,
             template_vip)
 
-        if ip.version == 4:
+        # keepalived will handle the VIP if we are on active/standby
+        if (ip.version == 4 and
+            CONF.controller_worker.loadbalancer_topology ==
+                consts.TOPOLOGY_SINGLE):
             # Create an IPv4 alias interface, needed in RH based flavors
             alias_interface_file_path = self.get_alias_network_interface_file(
                 primary_interface)
@@ -391,13 +405,16 @@ class RH(BaseOS):
             routes_interface_file_path, primary_interface,
             render_host_routes, template_routes, gateway, vip, netmask)
 
-        route_rules_interface_file_path = (
-            self.get_route_rules_interface_file(primary_interface))
-        template_rules = j2_env.get_template(self.RULE_ETH_X_CONF)
+        # keepalived will handle the rule(s) if we are on actvie/standby
+        if (CONF.controller_worker.loadbalancer_topology ==
+                consts.TOPOLOGY_SINGLE):
+            route_rules_interface_file_path = (
+                self.get_route_rules_interface_file(primary_interface))
+            template_rules = j2_env.get_template(self.RULE_ETH_X_CONF)
 
-        self.write_static_routes_interface_file(
-            route_rules_interface_file_path, primary_interface,
-            render_host_routes, template_rules, gateway, vip, netmask)
+            self.write_static_routes_interface_file(
+                route_rules_interface_file_path, primary_interface,
+                render_host_routes, template_rules, gateway, vip, netmask)
 
     def write_static_routes_interface_file(self, interface_file_path,
                                            interface, host_routes,
@@ -416,11 +433,13 @@ class RH(BaseOS):
         with os.fdopen(os.open(interface_file_path, flags, mode),
                        'w') as text_file:
             text = template_routes.render(
+                consts=consts,
                 interface=interface,
                 host_routes=host_routes,
                 gateway=gateway,
                 network=utils.ip_netmask_to_cidr(vip, netmask),
                 vip=vip,
+                topology=CONF.controller_worker.loadbalancer_topology,
             )
             text_file.write(text)
 
@@ -449,14 +468,15 @@ class RH(BaseOS):
                 routes_interface_file_path, netns_interface,
                 host_routes, template_routes, None, None, None)
 
-    def bring_interfaces_up(self, ip, primary_interface, secondary_interface):
+    @classmethod
+    def bring_interfaces_up(cls, ip, primary_interface, secondary_interface):
         if ip.version == 4:
-            super(RH, self).bring_interfaces_up(
+            super(RH, cls).bring_interfaces_up(
                 ip, primary_interface, secondary_interface)
         else:
             # Secondary interface is not present in IPv6 configuration
-            self._bring_if_down(primary_interface)
-            self._bring_if_up(primary_interface, 'VIP')
+            cls._bring_if_down(primary_interface)
+            cls._bring_if_up(primary_interface, 'VIP')
 
     def has_ifup_all(self):
         return False
