@@ -38,7 +38,11 @@ def get_hostname():
 
 
 def base64_sha1_string(string_to_hash):
-    hash_str = hashlib.sha1(string_to_hash.encode('utf-8')).digest()
+    """Get a b64-encoded sha1 hash of a string. Not intended to be secure!"""
+    # TODO(rm_work): applying nosec here because this is not intended to be
+    # secure, it's just a way to get a consistent ID. Changing this would
+    # break backwards compatibility with existing loadbalancers.
+    hash_str = hashlib.sha1(string_to_hash.encode('utf-8')).digest()  # nosec
     b64_str = base64.b64encode(hash_str, str.encode('_-', 'ascii'))
     return b64_str.decode('UTF-8')
 
@@ -78,6 +82,15 @@ def netmask_to_prefix(netmask):
     return netaddr.IPAddress(netmask).netmask_bits()
 
 
+def ip_netmask_to_cidr(ip, netmask):
+    net = netaddr.IPNetwork("0.0.0.0/0")
+    if ip and netmask:
+        net = netaddr.IPNetwork(
+            "{ip}/{netmask}".format(ip=ip, netmask=netmask)
+        )
+    return "{ip}/{netmask}".format(ip=net.network, netmask=net.prefixlen)
+
+
 class exception_logger(object):
     """Wrap a function and log raised exception
 
