@@ -45,6 +45,7 @@ class BaseDatabaseTask(task.Task):
         self.health_mon_repo = repo.HealthMonitorRepository()
         self.listener_repo = repo.ListenerRepository()
         self.loadbalancer_repo = repo.LoadBalancerRepository()
+        self.vip_repo = repo.VipRepository()
         self.member_repo = repo.MemberRepository()
         self.pool_repo = repo.PoolRepository()
         self.amp_health_repo = repo.AmphoraHealthRepository()
@@ -479,7 +480,7 @@ class AssociateFailoverAmphoraWithLBID(BaseDatabaseTask):
 class MapLoadbalancerToAmphora(BaseDatabaseTask):
     """Maps and assigns a load balancer to an amphora in the database."""
 
-    def execute(self, loadbalancer_id):
+    def execute(self, loadbalancer_id, server_group_id=None):
         """Allocates an Amphora for the load balancer in the database.
 
         :param loadbalancer_id: The load balancer id to map to an amphora
@@ -490,6 +491,11 @@ class MapLoadbalancerToAmphora(BaseDatabaseTask):
         LOG.debug("Allocating an Amphora for load balancer with id %s",
                   loadbalancer_id)
 
+        if server_group_id is not None:
+            LOG.debug("Load balancer is using anti-affinity. Skipping spares "
+                      "pool allocation.")
+            return None
+
         amp = self.amphora_repo.allocate_and_associate(
             db_apis.get_session(),
             loadbalancer_id)
@@ -867,8 +873,10 @@ class UpdateAmphoraInfo(BaseDatabaseTask):
         :param compute_obj: Compute on which an amphora resides
         :returns: Updated amphora object
         """
-        self.amphora_repo.update(db_apis.get_session(), amphora_id,
-                                 lb_network_ip=compute_obj.lb_network_ip)
+        self.amphora_repo.update(
+            db_apis.get_session(), amphora_id,
+            lb_network_ip=compute_obj.lb_network_ip,
+            cached_zone=compute_obj.cached_zone)
         return self.amphora_repo.get(db_apis.get_session(), id=amphora_id)
 
 
@@ -906,24 +914,6 @@ class UpdateAmphoraCertBusyToFalse(BaseDatabaseTask):
                                  cert_busy=False)
 
 
-class TestLBStatusSetPendingInDB(BaseDatabaseTask):
-    """Test the LB's status to get a lock, and set it to PENDING_UPDATE
-
-    """
-
-    def execute(self, loadbalancer_id):
-        LOG.debug("Attempting to get a lock for loadbalancer %s to set "
-                  "PENDING_UPDATE status", loadbalancer_id)
-        self.loadbalancer_repo.test_and_set_provisioning_status(
-            db_apis.get_session(), loadbalancer_id,
-            status=constants.PENDING_UPDATE, raise_exception=True)
-        LOG.debug("Set loadbalancer %s to PENDING_UPDATE", loadbalancer_id)
-
-    def revert(self, loadbalancer_id, *args, **kwargs):
-        LOG.debug("Marking loadbalancer %s ERROR", loadbalancer_id)
-        self.task_utils.mark_loadbalancer_prov_status_error(loadbalancer_id)
-
-
 class MarkLBActiveInDB(BaseDatabaseTask):
     """Mark the load balancer active in the DB.
 
@@ -1283,6 +1273,11 @@ class UpdateLoadbalancerInDB(BaseDatabaseTask):
         """
 
         LOG.debug("Update DB for loadbalancer id: %s ", loadbalancer.id)
+        if update_dict.get('vip'):
+            vip_dict = update_dict.pop('vip')
+            self.vip_repo.update(db_apis.get_session(),
+                                 loadbalancer.vip.load_balancer_id,
+                                 **vip_dict)
         self.loadbalancer_repo.update(db_apis.get_session(), loadbalancer.id,
                                       **update_dict)
 
@@ -1429,8 +1424,9 @@ class UpdatePoolInDB(BaseDatabaseTask):
 
         LOG.warning("Reverting update pool in DB for pool id %s", pool.id)
         try:
-            self.repos.update_pool_and_sp(db_apis.get_session(), pool.id,
-                                          provisioning_status=constants.ERROR)
+            self.repos.update_pool_and_sp(
+                db_apis.get_session(), pool.id,
+                dict(provisioning_status=constants.ERROR))
         except Exception as e:
             LOG.error("Failed to update pool %(pool)s provisioning_status to "
                       "ERROR due to: %(except)s", {'pool': pool.id,
@@ -1529,6 +1525,25 @@ class GetAmphoraDetails(BaseDatabaseTask):
                                    vrrp_priority=amphora.vrrp_priority)
 
 
+class GetAmphoraeFromLoadbalancer(BaseDatabaseTask):
+    """Task to pull the listeners from a loadbalancer."""
+
+    def execute(self, loadbalancer):
+        """Pull the amphorae from a loadbalancer.
+
+        :param loadbalancer: Load balancer which listeners are required
+        :returns: A list of Listener objects
+        """
+        amphorae = []
+        for amp in loadbalancer.amphorae:
+            a = self.amphora_repo.get(db_apis.get_session(), id=amp.id)
+
+            if a is None or a.status == constants.DELETED:
+                continue
+            amphorae.append(a)
+        return amphorae
+
+
 class GetListenersFromLoadbalancer(BaseDatabaseTask):
     """Task to pull the listeners from a loadbalancer."""
 
@@ -1541,6 +1556,7 @@ class GetListenersFromLoadbalancer(BaseDatabaseTask):
         listeners = []
         for listener in loadbalancer.listeners:
             l = self.listener_repo.get(db_apis.get_session(), id=listener.id)
+            l.load_balancer = loadbalancer
             listeners.append(l)
         return listeners
 
@@ -2635,3 +2651,25 @@ class CountPoolChildrenForQuota(BaseDatabaseTask):
         member_count = len(pool.members)
 
         return {'HM': health_mon_count, 'member': member_count}
+
+
+class UpdatePoolMembersOperatingStatusInDB(BaseDatabaseTask):
+    """Updates the members of a pool operating status.
+
+    Since sqlalchemy will likely retry by itself always revert if it fails
+    """
+
+    def execute(self, pool, operating_status):
+        """Update the members of a pool operating status in DB.
+
+        :param pool: Pool object to be updated
+        :param operating_status: Operating status to set
+        :returns: None
+        """
+
+        LOG.debug("Updating member operating status to %(status)s in DB for "
+                  "pool id: %(pool)s", {'status': operating_status,
+                                        'pool': pool.id})
+        self.member_repo.update_pool_members(db_apis.get_session(),
+                                             pool.id,
+                                             operating_status=operating_status)
