@@ -406,7 +406,8 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
                           tls=False, sni=False, peer_port=None, topology=None,
                           l7=False, enabled=True, insert_headers=None,
                           be_proto=None, monitor_ip_port=False,
-                          monitor_proto=None, connection_limit=-1):
+                          monitor_proto=None, connection_limit=-1,
+                          disabled_member=False):
     proto = 'HTTP' if proto is None else proto
     if be_proto is None:
         be_proto = 'HTTP' if proto is 'TERMINATED_HTTPS' else proto
@@ -445,7 +446,8 @@ def sample_listener_tuple(proto=None, monitor=True, persistence=True,
                 proto=be_proto, monitor=monitor, persistence=persistence,
                 persistence_type=persistence_type,
                 persistence_cookie=persistence_cookie,
-                monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto)]
+                monitor_ip_port=monitor_ip_port, monitor_proto=monitor_proto,
+                disabled_member=disabled_member)]
         l7policies = []
     return in_listener(
         id='sample_listener_id_1',
@@ -517,7 +519,7 @@ def sample_tls_container_tuple(id='cont_id_1', certificate=None,
 def sample_pool_tuple(proto=None, monitor=True, persistence=True,
                       persistence_type=None, persistence_cookie=None,
                       sample_pool=1, monitor_ip_port=False,
-                      monitor_proto=None):
+                      monitor_proto=None, disabled_member=False):
     proto = 'HTTP' if proto is None else proto
     monitor_proto = proto if monitor_proto is None else monitor_proto
     in_pool = collections.namedtuple(
@@ -532,7 +534,8 @@ def sample_pool_tuple(proto=None, monitor=True, persistence=True,
         members = [sample_member_tuple('sample_member_id_1', '10.0.0.99',
                                        monitor_ip_port=monitor_ip_port),
                    sample_member_tuple('sample_member_id_2', '10.0.0.98',
-                                       monitor_ip_port=monitor_ip_port)]
+                                       monitor_ip_port=monitor_ip_port,
+                                       enabled=not disabled_member)]
         if monitor is True:
             mon = sample_health_monitor_tuple(proto=monitor_proto)
     elif sample_pool == 2:
