@@ -56,10 +56,20 @@ _l7rule_mock = mock.MagicMock()
 _create_map_flow_mock = mock.MagicMock()
 _amphora_mock.load_balancer_id = LB_ID
 _amphora_mock.id = AMP_ID
+_db_session = mock.MagicMock()
 
 CONF = cfg.CONF
 
 
+class TestException(Exception):
+
+    def __init__(self, value):
+        self.value = value
+
+    def __str__(self):
+        return repr(self.value)
+
+
 @mock.patch('octavia.db.repositories.AmphoraRepository.get',
             return_value=_amphora_mock)
 @mock.patch('octavia.db.repositories.HealthMonitorRepository.get',
@@ -79,7 +89,7 @@ CONF = cfg.CONF
 @mock.patch('octavia.common.base_taskflow.BaseTaskFlowEngine._taskflow_load',
             return_value=_flow_mock)
 @mock.patch('taskflow.listeners.logging.DynamicLoggingListener')
-@mock.patch('octavia.db.api.get_session', return_value='TEST')
+@mock.patch('octavia.db.api.get_session', return_value=_db_session)
 class TestControllerWorker(base.TestCase):
 
     def setUp(self):
@@ -89,7 +99,6 @@ class TestControllerWorker(base.TestCase):
         _pool_mock.listeners = [_listener_mock]
         _pool_mock.load_balancer = _load_balancer_mock
         _health_mon_mock.pool = _pool_mock
-        self.amp_mock = _amphora_mock
         _load_balancer_mock.amphorae = _amphora_mock
         _load_balancer_mock.vip = _vip_mock
         _listener_mock.load_balancer = _load_balancer_mock
@@ -163,7 +172,7 @@ class TestControllerWorker(base.TestCase):
         cw.delete_amphora(AMP_ID)
 
         mock_amp_repo_get.assert_called_once_with(
-            'TEST',
+            _db_session,
             id=AMP_ID)
 
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
@@ -577,7 +586,7 @@ class TestControllerWorker(base.TestCase):
         cw.delete_load_balancer(LB_ID, cascade=False)
 
         mock_lb_repo_get.assert_called_once_with(
-            'TEST',
+            _db_session,
             id=LB_ID)
 
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
@@ -615,7 +624,7 @@ class TestControllerWorker(base.TestCase):
         cw.delete_load_balancer(LB_ID, cascade=True)
 
         mock_lb_repo_get.assert_called_once_with(
-            'TEST',
+            _db_session,
             id=LB_ID)
 
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
@@ -657,7 +666,7 @@ class TestControllerWorker(base.TestCase):
         cw.update_load_balancer(LB_ID, change)
 
         mock_lb_repo_get.assert_called_once_with(
-            'TEST',
+            _db_session,
             id=LB_ID)
 
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
@@ -774,6 +783,38 @@ class TestControllerWorker(base.TestCase):
 
         _flow_mock.run.assert_called_once_with()
 
+    @mock.patch('octavia.controller.worker.flows.'
+                'member_flows.MemberFlows.get_batch_update_members_flow',
+                return_value=_flow_mock)
+    def test_batch_update_members(self,
+                                  mock_get_batch_update_members_flow,
+                                  mock_api_get_session,
+                                  mock_dyn_log_listener,
+                                  mock_taskflow_load,
+                                  mock_pool_repo_get,
+                                  mock_member_repo_get,
+                                  mock_l7rule_repo_get,
+                                  mock_l7policy_repo_get,
+                                  mock_listener_repo_get,
+                                  mock_lb_repo_get,
+                                  mock_health_mon_repo_get,
+                                  mock_amp_repo_get):
+
+        _flow_mock.reset_mock()
+
+        cw = controller_worker.ControllerWorker()
+        cw.batch_update_members([9], [11], [MEMBER_UPDATE_DICT])
+
+        (base_taskflow.BaseTaskFlowEngine._taskflow_load.
+            assert_called_once_with(_flow_mock,
+                                    store={
+                                        constants.LISTENERS: [_listener_mock],
+                                        constants.LOADBALANCER:
+                                            _load_balancer_mock,
+                                        constants.POOL: _pool_mock}))
+
+        _flow_mock.run.assert_called_once_with()
+
     @mock.patch('octavia.controller.worker.flows.'
                 'pool_flows.PoolFlows.get_create_pool_flow',
                 return_value=_flow_mock)
@@ -995,6 +1036,7 @@ class TestControllerWorker(base.TestCase):
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
             assert_called_once_with(_flow_mock,
                                     store={constants.L7RULE: _l7rule_mock,
+                                           constants.L7POLICY: _l7policy_mock,
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
@@ -1027,6 +1069,7 @@ class TestControllerWorker(base.TestCase):
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
             assert_called_once_with(_flow_mock,
                                     store={constants.L7RULE: _l7rule_mock,
+                                           constants.L7POLICY: _l7policy_mock,
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
@@ -1059,6 +1102,7 @@ class TestControllerWorker(base.TestCase):
         (base_taskflow.BaseTaskFlowEngine._taskflow_load.
             assert_called_once_with(_flow_mock,
                                     store={constants.L7RULE: _l7rule_mock,
+                                           constants.L7POLICY: _l7policy_mock,
                                            constants.LISTENERS:
                                                [_listener_mock],
                                            constants.LOADBALANCER:
@@ -1071,8 +1115,10 @@ class TestControllerWorker(base.TestCase):
     @mock.patch('octavia.controller.worker.flows.'
                 'amphora_flows.AmphoraFlows.get_failover_flow',
                 return_value=_flow_mock)
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
     def test_failover_amphora(self,
-                              mock_get_update_listener_flow,
+                              mock_update,
+                              mock_get_failover_flow,
                               mock_api_get_session,
                               mock_dyn_log_listener,
                               mock_taskflow_load,
@@ -1101,6 +1147,80 @@ class TestControllerWorker(base.TestCase):
                        }))
 
         _flow_mock.run.assert_called_once_with()
+        mock_update.assert_called_with(_db_session, LB_ID,
+                                       provisioning_status=constants.ACTIVE)
+
+    @mock.patch('octavia.controller.worker.controller_worker.ControllerWorker.'
+                '_perform_amphora_failover')
+    def test_failover_amp_missing_amp(self,
+                                      mock_perform_amp_failover,
+                                      mock_api_get_session,
+                                      mock_dyn_log_listener,
+                                      mock_taskflow_load,
+                                      mock_pool_repo_get,
+                                      mock_member_repo_get,
+                                      mock_l7rule_repo_get,
+                                      mock_l7policy_repo_get,
+                                      mock_listener_repo_get,
+                                      mock_lb_repo_get,
+                                      mock_health_mon_repo_get,
+                                      mock_amp_repo_get):
+
+        mock_amp_repo_get.return_value = None
+
+        cw = controller_worker.ControllerWorker()
+        cw.failover_amphora(AMP_ID)
+
+        mock_perform_amp_failover.assert_not_called()
+
+    @mock.patch('octavia.controller.worker.controller_worker.ControllerWorker.'
+                '_perform_amphora_failover')
+    def test_failover_amp_flow_exception(self,
+                                         mock_perform_amp_failover,
+                                         mock_api_get_session,
+                                         mock_dyn_log_listener,
+                                         mock_taskflow_load,
+                                         mock_pool_repo_get,
+                                         mock_member_repo_get,
+                                         mock_l7rule_repo_get,
+                                         mock_l7policy_repo_get,
+                                         mock_listener_repo_get,
+                                         mock_lb_repo_get,
+                                         mock_health_mon_repo_get,
+                                         mock_amp_repo_get):
+
+        mock_perform_amp_failover.side_effect = TestException('boom')
+        cw = controller_worker.ControllerWorker()
+        self.assertRaises(TestException, cw.failover_amphora, AMP_ID)
+
+    @mock.patch('octavia.controller.worker.controller_worker.ControllerWorker.'
+                '_perform_amphora_failover')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
+    def test_failover_amp_no_lb(self,
+                                mock_lb_update,
+                                mock_perform_amp_failover,
+                                mock_api_get_session,
+                                mock_dyn_log_listener,
+                                mock_taskflow_load,
+                                mock_pool_repo_get,
+                                mock_member_repo_get,
+                                mock_l7rule_repo_get,
+                                mock_l7policy_repo_get,
+                                mock_listener_repo_get,
+                                mock_lb_repo_get,
+                                mock_health_mon_repo_get,
+                                mock_amp_repo_get):
+
+        amphora = mock.MagicMock()
+        amphora.load_balancer_id = None
+        mock_amp_repo_get.return_value = amphora
+
+        cw = controller_worker.ControllerWorker()
+        cw.failover_amphora(AMP_ID)
+
+        mock_lb_update.assert_not_called()
+        mock_perform_amp_failover.assert_called_once_with(
+            amphora, constants.LB_CREATE_FAILOVER_PRIORITY)
 
     @mock.patch('octavia.db.repositories.AmphoraHealthRepository.update')
     def test_failover_deleted_amphora(self,
@@ -1116,25 +1236,76 @@ class TestControllerWorker(base.TestCase):
                                       mock_lb_repo_get,
                                       mock_health_mon_repo_get,
                                       mock_amp_repo_get):
+
         mock_taskflow_load.reset_mock()
-        self.amp_mock.id = AMP_ID
-        self.amp_mock.status = constants.DELETED
+        mock_amphora = mock.MagicMock()
+        mock_amphora.id = AMP_ID
+        mock_amphora.status = constants.DELETED
 
         cw = controller_worker.ControllerWorker()
-        cw.failover_amphora(AMP_ID)
+        cw._perform_amphora_failover(mock_amphora, 10)
 
-        mock_update.assert_called_with('TEST', AMP_ID, busy=True)
+        mock_update.assert_called_with(_db_session, AMP_ID, busy=True)
         mock_taskflow_load.assert_not_called()
 
+    @mock.patch('octavia.controller.worker.'
+                'controller_worker.ControllerWorker._perform_amphora_failover')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
+    def test_failover_loadbalancer(self,
+                                   mock_update,
+                                   mock_perform,
+                                   mock_api_get_session,
+                                   mock_dyn_log_listener,
+                                   mock_taskflow_load,
+                                   mock_pool_repo_get,
+                                   mock_member_repo_get,
+                                   mock_l7rule_repo_get,
+                                   mock_l7policy_repo_get,
+                                   mock_listener_repo_get,
+                                   mock_lb_repo_get,
+                                   mock_health_mon_repo_get,
+                                   mock_amp_repo_get):
+        _amphora_mock2 = mock.MagicMock()
+        _amphora_mock3 = mock.MagicMock()
+        _amphora_mock3.status = constants.DELETED
+        _load_balancer_mock.amphorae = [
+            _amphora_mock, _amphora_mock2, _amphora_mock3]
+        cw = controller_worker.ControllerWorker()
+        cw.failover_loadbalancer('123')
+        mock_perform.assert_called_with(
+            _amphora_mock2, constants.LB_CREATE_ADMIN_FAILOVER_PRIORITY)
+        mock_update.assert_called_with(_db_session, '123',
+                                       provisioning_status=constants.ACTIVE)
+
+        mock_perform.reset
+        _load_balancer_mock.amphorae = [
+            _amphora_mock, _amphora_mock2, _amphora_mock3]
+        _amphora_mock2.role = constants.ROLE_BACKUP
+        cw.failover_loadbalancer('123')
+        # because mock2 gets failed over earlier now _amphora_mock
+        # is the last one
+        mock_perform.assert_called_with(
+            _amphora_mock, constants.LB_CREATE_ADMIN_FAILOVER_PRIORITY)
+        mock_update.assert_called_with(_db_session, '123',
+                                       provisioning_status=constants.ACTIVE)
+
+        mock_perform.reset
+        mock_perform.side_effect = OverflowError()
+        self.assertRaises(OverflowError, cw.failover_loadbalancer, 123)
+        mock_update.assert_called_with(_db_session, 123,
+                                       provisioning_status=constants.ERROR)
+
     @mock.patch('octavia.controller.worker.flows.'
                 'amphora_flows.AmphoraFlows.get_failover_flow',
                 return_value=_flow_mock)
     @mock.patch(
-        'octavia.db.repositories.AmphoraRepository.get_all_lbs_on_amphora',
-        return_value=[_load_balancer_mock])
+        'octavia.db.repositories.AmphoraRepository.get_lb_for_amphora',
+        return_value=_load_balancer_mock)
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.update')
     def test_failover_amphora_anti_affinity(self,
+                                            mock_update,
                                             mock_get_update_listener_flow,
-                                            mock_get_all_lbs_for_amp_mock,
+                                            mock_get_lb_for_amphora,
                                             mock_api_get_session,
                                             mock_dyn_log_listener,
                                             mock_taskflow_load,
@@ -1166,6 +1337,8 @@ class TestControllerWorker(base.TestCase):
                        }))
 
         _flow_mock.run.assert_called_once_with()
+        mock_update.assert_called_with(_db_session, LB_ID,
+                                       provisioning_status=constants.ACTIVE)
 
     @mock.patch('octavia.controller.worker.flows.'
                 'amphora_flows.AmphoraFlows.cert_rotate_amphora_flow',
